@@ -10,6 +10,7 @@ no warnings 'redefine';
 use OpenGL qw/ :glfunctions :glconstants gluPerspective gluOrtho2D /;
 use OpenGL::GLUT qw( :all );
 use PDL::Graphics::OpenGL::Perl::OpenGL;
+use PDL::Core qw(barf);
 
 $PDL::Graphics::TriD::create_window_sub = # warnings
 $PDL::Graphics::TriD::create_window_sub = sub {
@@ -37,75 +38,58 @@ sub PDL::Graphics::TriD::Object::cannot_mklist {
 
 sub PDL::Graphics::TriD::Object::gl_update_list {
   my($this) = @_;
-  if($this->{List}) {
-	 glDeleteLists($this->{List},1);
-  }
-  my $lno = glGenLists(1);
-  $this->{List} = $lno;
+  glDeleteLists($this->{List},1) if $this->{List};
+  $this->{List} = my $lno = glGenLists(1);
   print "GENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
   glNewList($lno,GL_COMPILE);
-  if ($PDL::Graphics::TriD::any_cannots) {
-	 for(@{$this->{Objects}}) {
-		if(!$_->cannot_mklist()) {
-		  $_->togl();
-		}
-	 }
-  } else { 
-	 for (@{$this->{Objects}}) {
-		$_->togl()
-	 } 
-  }
-  print "EGENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
-  #	pdltotrianglemesh($pdl, 0, 1, 0, ($pdl->{Dims}[1]-1)*$mult);
-  glEndList();
+  eval {
+    my @objs = @{$this->{Objects}};
+    @objs = grep !$_->cannot_mklist(), @objs if $PDL::Graphics::TriD::any_cannots;
+    $_->togl() for @objs;
+    print "EGENLIST $lno\n" if($PDL::Graphics::TriD::verbose);
+    #	pdltotrianglemesh($pdl, 0, 1, 0, ($pdl->{Dims}[1]-1)*$mult);
+  };
+  { local $@; glEndList(); }
+  die if $@;
   print "VALID1 $this\n" if($PDL::Graphics::TriD::verbose);
   $this->{ValidList} = 1;
 }
 
 sub PDL::Graphics::TriD::Object::gl_call_list {
 	my($this) = @_;
-	print "CALLIST ",$this->{List}//'undef',"!\n" if($PDL::Graphics::TriD::verbose);
-	print "CHECKVALID $this\n" if($PDL::Graphics::TriD::verbose);
+	print "CALLIST ",$this->{List}//'undef',"!\n" if $PDL::Graphics::TriD::verbose;
+	print "CHECKVALID $this=$this->{ValidList}\n" if $PDL::Graphics::TriD::verbose;
 	$this->gl_update_list if !$this->{ValidList};
 	glCallList($this->{List});
-	if ($PDL::Graphics::TriD::any_cannots) {
-	  for(@{$this->{Objects}}) {
-		if($_->cannot_mklist()) {
-			print ref($_)," cannot mklist\n";
-			$_->togl();
-		}
-	  }
-        }
+	return if !$PDL::Graphics::TriD::any_cannots;
+	for (grep $_->cannot_mklist, @{$this->{Objects}}) {
+	  print ref($_)," cannot mklist\n";
+	  $_->togl();
+	}
 }
 
 sub PDL::Graphics::TriD::Object::delete_displist {
 	my($this) = @_;
-	if($this->{List}) {
-		glDeleteLists($this->{List},1);
-		undef $this->{List};
-	}
+	return if !$this->{List};
+	glDeleteLists($this->{List},1);
+	delete @$this{qw(List ValidList)};
 }
 
-sub PDL::Graphics::TriD::Object::togl {
-	my($this) = @_;
-	for(@{$this->{Objects}}) { $_->togl() }
-}
+sub PDL::Graphics::TriD::Object::togl { $_->togl for @{$_[0]->{Objects}} }
 
+my @bb1 = ([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
+	   [3,1,2],[3,1,5],[3,4,5],[3,4,2]);
+my @bb2 = ([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]);
 sub PDL::Graphics::TriD::BoundingBox::togl { 
   my($this) = @_;
   $this = $this->{Box};
   glDisable(GL_LIGHTING);
   glColor3d(1,1,1);
   glBegin(GL_LINES);
-  for([0,4,2],[0,1,2],[0,1,5],[0,4,5],[0,4,2],[3,4,2],
-		[3,1,2],[3,1,5],[3,4,5],[3,4,2]) {
-	 glVertex3d(@{$this}[@$_]);
-  }
+  glVertex3d(@{$this}[@$_]) for @bb1;
   glEnd();
   glBegin(GL_LINE_STRIP);
-  for([0,1,2],[3,1,2],[0,1,5],[3,1,5],[0,4,5],[3,4,5]) {
-	 glVertex3d(@{$this}[@$_]);
-  }
+  glVertex3d(@{$this}[@$_]) for @bb2;
   glEnd();
   glEnable(GL_LIGHTING);
 }
@@ -177,17 +161,11 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 	$_ = $_->dup(1,3)->rotate($dupseq) for $starts, $ends;
 	$line_coord = $line_coord->glue(1, $starts->append($ends));
 	my $axisvals = zeroes(3,$ndiv+1)->ylinvals($this->{Scale}->dog)->transpose->flat->transpose;
-	my @label = map [@$_[0..2], sprintf "%.3f", $_->[3]], @{ $ends->append($axisvals)->unpdl };
-	my $dim = 0; push @label, map [@$_, $this->{Names}[$dim++]], @{ ($id3*1.1)->unpdl };
+	my @label = map sprintf("%.3f", $_), @{ $axisvals->flat->unpdl };
+	push @label, @{$this->{Names}};
 	glColor3d(1,1,1);
-	for (@label) {
-		glRasterPos3f(@$_[0..2]);
-		if ( done_glutInit() ) {
-		   glutBitmapString($fontbase, $_->[3]);
-		} else {
-		   OpenGL::glpPrintString($fontbase, $_->[3]);
-		}
-	}
+	PDL::Graphics::OpenGLQ::gl_texts($ends->glue(1, $id3), done_glutInit(),
+	    $fontbase, \@label);
 	PDL::gl_lines_nc($line_coord->splitdim(0,3)->clump(1,2));
 	glEnable(GL_LIGHTING);
 }
@@ -216,53 +194,66 @@ sub PDL::Graphics::TriD::GObject::togl_graph {
 
 sub PDL::Graphics::TriD::Points::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_points_col($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
-}
-
-sub PDL::gl_spheres {
-   my ($coords,$colors) = @_;   
-   for (my $np=0; $np<$coords->dim(1); $np++) {
-      glPushMatrix();
-      my ($x,$y,$z) = ($coords->slice(":,($np)"))->float->list;
-      glTranslatef($x,$y,$z);
-      glutSolidSphere(0.025,15,15);
-      glPopMatrix();
-   }
+	eval {
+	  PDL::gl_points_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::Spheres::gdraw {
    my($this,$points) = @_;
-   $this->glOptions();
+   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+   $this->glOptions;
+   glEnable(GL_LIGHTING);
    glShadeModel(GL_SMOOTH);
-   PDL::gl_spheres($points,$this->{Colors});
+   eval {
+      PDL::gl_spheres($points, 0.025, 15, 15);
+   };
+   { local $@; glPopAttrib(); }
+   die if $@;
 }
 
 sub PDL::Graphics::TriD::Lattice::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	barf "Need 3D points AND colours"
+	  if grep $_->ndims < 3, $points, $this->{Colors};
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_line_strip_col($points,$this->{Colors});
-	PDL::gl_line_strip_col($points->xchg(1,2),$this->{Colors}->xchg(1,2));
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_line_strip_col($points,$this->{Colors});
+	  PDL::gl_line_strip_col($points->xchg(1,2),$this->{Colors}->xchg(1,2));
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::LineStrip::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_line_strip_col($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_line_strip_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::Lines::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
-	PDL::gl_lines_col($points,$this->{Colors});
-	glEnable(GL_LIGHTING);
+	eval {
+	  PDL::gl_lines_col($points,$this->{Colors});
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::GObject::glOptions {
@@ -271,189 +262,172 @@ sub PDL::Graphics::TriD::GObject::glOptions {
   glPointSize($this->{Options}{PointSize} || 1);
 }
 
+sub PDL::Graphics::TriD::GObject::_lattice_lines {
+  my ($this, $points) = @_;
+  glDisable(GL_LIGHTING);
+  glColor3f(0,0,0);
+  PDL::gl_line_strip_nc($points);
+  PDL::gl_line_strip_nc($points->xchg(1,2));
+}
+
 sub PDL::Graphics::TriD::Contours::gdraw {
   my($this,$points) = @_;
-  $this->glOptions();
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+  $this->glOptions;
   glDisable(GL_LIGHTING);
-  my $pcnt=0;
-  my $i=0;
-  foreach (grep defined, @{$this->{ContourSegCnt}}){
-	my $colors =  $this->{Colors};
-	$colors = $colors->slice(":,($i)") if $colors->getndims==2;
-	PDL::gl_lines_col($points->slice(":,$pcnt:$_"),$colors);
-	$i++;
-	$pcnt=$_+1;
+  eval {
+    my $pcnt=0;
+    my $i=0;
+    foreach (grep defined, @{$this->{ContourSegCnt}}){
+	  my $colors =  $this->{Colors};
+	  $colors = $colors->slice(":,($i)") if $colors->getndims==2;
+	  PDL::gl_lines_col($points->slice(":,$pcnt:$_"),$colors);
+	  $i++;
+	  $pcnt=$_+1;
+    }
+    if(defined $this->{Labels}){
+	   glColor3d(1,1,1);
+	   my $seg = sprintf(":,%d:%d",$this->{Labels}[0],$this->{Labels}[1]);
+	   PDL::Graphics::OpenGLQ::gl_texts($points->slice($seg),
+		   done_glutInit(), $this->{Options}{Font},
+		   $this->{LabelStrings});
+    }
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
+}
+
+my @sls1 = (
+  ":,0:-2,0:-2",
+  ":,1:-1,0:-2",
+  ":,0:-2,1:-1");
+my @sls2 = (
+  ":,1:-1,1:-1",
+  ":,0:-2,1:-1",
+  ":,1:-1,0:-2");
+sub _lattice_slice {
+  my ($f, @pdls) = @_;
+  for my $s (\@sls1, \@sls2) {
+    my @args;
+    for my $p (@pdls) {
+      push @args, map $p->slice($_), @$s;
+    }
+    &$f(@args);
   }
-  if(defined $this->{Labels}){
-	 glColor3d(1,1,1);
-	 my $seg = sprintf(":,%d:%d",$this->{Labels}[0],$this->{Labels}[1]);
-	 PDL::Graphics::OpenGLQ::gl_texts($points->slice($seg),
-		 $this->{Options}{Font}
-		 ,$this->{LabelStrings});
-  }
-  glEnable(GL_LIGHTING);
 }
 
 sub PDL::Graphics::TriD::SLattice::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (GL_SMOOTH);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls1),
-		(map {$this->{Colors}->slice($_)} @sls1)
-	);
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls2),
-		(map {$this->{Colors}->slice($_)} @sls2)
-	);
-	if ($this->{Options}{Lines}) {
-	  glColor3f(0,0,0);
-	  PDL::gl_line_strip_nc($points);
-	  PDL::gl_line_strip_nc($points->xchg(1,2));
-	}
-	glPopAttrib();
+	glShadeModel(GL_SMOOTH);
+	eval {
+	  _lattice_slice(\&PDL::gl_triangles, $points, $this->{Colors});
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SCLattice::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 	glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
 	glShadeModel(GL_FLAT);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls1),
-		(map {$this->{Colors}} @sls1)
-	);
-	PDL::gl_triangles(
-		(map {$points->slice($_)} @sls2),
-		(map {$this->{Colors}} @sls2)
-	);
-	if ($this->{Options}{Lines}) {
-	  glColor3f(0,0,0);
-	  PDL::gl_line_strip_nc($points);
-	  PDL::gl_line_strip_nc($points->xchg(1,2));
-	}
-	glPopAttrib();
+	eval {
+	  _lattice_slice(\&PDL::gl_triangles, $points, $this->{Colors});
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SLattice_S::gdraw {
 	my($this,$points) = @_;
-	$this->glOptions();
+	barf "Need 3D points"
+	  if grep $_->ndims < 3, $points;
 	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+	$this->glOptions;
 # For some reason, we need to set this here as well.
 	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
 # By-vertex doesn't make sense otherwise.
-	glShadeModel (GL_SMOOTH);
-	my @sls1 = (":,0:-2,0:-2",
-	            ":,1:-1,0:-2",
-		    ":,0:-2,1:-1");
-	my @sls2 = (":,1:-1,1:-1",
-		    ":,0:-2,1:-1",
-	            ":,1:-1,0:-2"
-		    );
-	if ($this->{Options}{Smooth}) {
-	  $this->{Normals} = $this->smoothn($points)
-	    unless defined($this->{Normals});
-	  my $n = $this->{Normals};
-	  my $f = (!$this->{Options}{Material} ?
-	  		\&PDL::gl_triangles_wn : \&PDL::gl_triangles_wn_mat);
-	  &$f(
-			       (map {$points->slice($_)} @sls1),
-			       (map {$n->slice($_)} @sls1),
-			       (map {$this->{Colors}->slice($_)} @sls1)
-			      );
-	  &$f(
-			       (map {$points->slice($_)} @sls2),
-			       (map {$n->slice($_)} @sls2),
-			       (map {$this->{Colors}->slice($_)} @sls2)
-			      );
-	} else {
-	  my $f = (!$this->{Options}{Material} ?
-	  		\&PDL::gl_triangles_n : \&PDL::gl_triangles_n_mat);
-	  &$f(
-			      (map {$points->slice($_)} @sls1),
-			      (map {$this->{Colors}->slice($_)} @sls1)
-			     );
-	  &$f(
-			      (map {$points->slice($_)} @sls2),
-			      (map {$this->{Colors}->slice($_)} @sls2)
-			     );
-	}
-	glDisable(GL_LIGHTING);
-	if ($this->{Options}{Lines}) {
-	  glColor3f(0,0,0);
-	  PDL::gl_line_strip_nc($points);
-	  PDL::gl_line_strip_nc($points->xchg(1,2));
-	}
-	glPopAttrib();
+	glShadeModel(GL_SMOOTH);
+	eval {
+	  my $f = 'PDL::gl_triangles_';
+	  $f .= 'w' if $this->{Options}{Smooth};
+	  $f .= 'n_mat';
+	  { no strict 'refs'; $f = \&$f; }
+	  my @pdls = $points;
+	  push @pdls, $this->{Normals} if $this->{Options}{Smooth};
+	  push @pdls, $this->{Colors};
+	  _lattice_slice($f, @pdls);
+	  $this->_lattice_lines($points) if $this->{Options}{Lines};
+	};
+	{ local $@; glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::STrigrid_S::gdraw {
   my($this,$points) = @_;
   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
-  # For some reason, we need to set this here as well.
-  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-  # By-vertex doesn't make sense otherwise.
-  glShadeModel(GL_SMOOTH);   
-  my @sls = (":,(0)",":,(1)",":,(2)");
-  my $idx = [0,1,2,0]; # for lines, below
-  if ($this->{Options}{Smooth}) {
-    $this->{Normals} //= $this->smoothn($this->{Points});
-    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_wn
-                                      :\&PDL::gl_triangles_wn_mat);
-    my $tmpn=$this->{Normals}->dice_axis(1,$this->{Faceidx}->clump(-1))
-                    ->splitdim(1,($this->{Faceidx}->dims)[0]);
-    my @args=((map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-              (map {$tmpn->slice($_)} @sls),
-              (map {$this->{Colors}->slice($_)} @sls) );&$f(@args);
-  } else {
-    my $f=(!$this->{Options}{Material}?\&PDL::gl_triangles_n
-                                      :\&PDL::gl_triangles_n_mat);
-    &$f( (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-         (map {$this->{Colors}->slice($_)} @sls) );
-  }
-  glDisable(GL_LIGHTING);
-  if ($this->{Options}{Lines}) {
-    glColor3f(0,0,0);
-    PDL::gl_lines_nc($this->{Faces}->dice_axis(1,$idx));
-  }
-  glPopAttrib();
+  $this->glOptions;
+  eval {
+    # For some reason, we need to set this here as well.
+    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+    # By-vertex doesn't make sense otherwise.
+    glShadeModel(GL_SMOOTH);
+    my @sls = (":,(0)",":,(1)",":,(2)");
+    my $idx = [0,1,2,0]; # for lines, below
+    if ($this->{Options}{Smooth}) {
+      my $f=\&PDL::gl_triangles_wn_mat;
+      my $tmpn=$this->{Normals}->dice_axis(1,$this->{Faceidx}->clump(-1))
+		      ->splitdim(1,$this->{Faceidx}->dim(0));
+      my @args=((map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+		(map {$tmpn->slice($_)} @sls),
+		(map {$this->{Colors}->slice($_)} @sls) );&$f(@args);
+    } else {
+      my $f=\&PDL::gl_triangles_n_mat;
+      &$f( (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+	   (map {$this->{Colors}->slice($_)} @sls) );
+    }
+    if ($this->{Options}{Lines}) {
+      glDisable(GL_LIGHTING);
+      glColor3f(0,0,0);
+      PDL::gl_lines_nc($this->{Faces}->dice_axis(1,$idx));
+    }
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
 }
 
 sub PDL::Graphics::TriD::STrigrid::gdraw {
   my($this,$points) = @_;
   glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
-  glDisable(GL_LIGHTING);
+  $this->glOptions;
+  eval {
+    glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-  glShadeModel (GL_SMOOTH);
-  my @sls = (":,(0)",":,(1)",":,(2)");
-  my $idx = [0,1,2,0];
-  PDL::gl_triangles(
-    (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
-    (map {$this->{Colors}->slice($_)} @sls));
-  if ($this->{Options}{Lines}) {
-    glColor3f(0,0,0);
-    PDL::gl_lines_nc($this->{Faces}->dice_axis(1,$idx));
+    glShadeModel (GL_SMOOTH);
+    my @sls = (":,(0)",":,(1)",":,(2)");
+    my $idx = [0,1,2,0];
+    PDL::gl_triangles(
+      (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
+      (map {$this->{Colors}->slice($_)} @sls));
+    if ($this->{Options}{Lines}) {
+      glColor3f(0,0,0);
+      PDL::gl_lines_nc($this->{Faces}->dice_axis(1,$idx));
   }
-  glPopAttrib();
+  };
+  { local $@; glPopAttrib(); }
+  die if $@;
 }
 
 ##################################
@@ -477,6 +451,10 @@ sub PDL::Graphics::TriD::Image::togl_graph {
 sub PDL::Graphics::TriD::Image::gdraw {
 	my($this,$vert) = @_;
 	my ($p,$xd,$yd,$txd,$tyd) = $this->flatten(1); # do binary alignment
+	if(!defined $vert) {$vert = $this->{Points}}
+	barf "Need 3,4 vert"
+	  if grep $_->dim(1) < 4 || $_->dim(0) != 3, $vert;
+	glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
 	glColor3d(1,1,1);
          glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
 	 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
@@ -487,20 +465,20 @@ sub PDL::Graphics::TriD::Image::gdraw {
 	glNormal3d(0,0,1);
 	glEnable(GL_TEXTURE_2D);
 	glBegin(GL_QUADS);
-	my @texvert = (
-		[0,0],
-		[$xd/$txd, 0],
-		[$xd/$txd, $yd/$tyd],
-		[0, $yd/$tyd]
-	);
-	if(!defined $vert) {$vert = $this->{Points}}
-	for(0..3) {
-		glTexCoord2f(@{$texvert[$_]});
-		glVertex3f($vert->slice(":,($_)")->list);
-	}
-	glEnd();
-	glEnable(GL_LIGHTING);
-	glDisable(GL_TEXTURE_2D);
+	eval {
+	  my @texvert = (
+		  [0,0],
+		  [$xd/$txd, 0],
+		  [$xd/$txd, $yd/$tyd],
+		  [0, $yd/$tyd]
+	  );
+	  for(0..3) {
+		  glTexCoord2f(@{$texvert[$_]});
+		  glVertex3f($vert->slice(":,($_)")->list);
+	  }
+	};
+	{ local $@; glEnd(); glPopAttrib(); }
+	die if $@;
 }
 
 sub PDL::Graphics::TriD::SimpleController::togl {
@@ -523,9 +501,6 @@ use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/Ev Width Height Interactive _GLObject
               _ViewPorts _CurrentViewPort /;
 
-sub i_keep_list {return 1} # For Object, so I will be notified of changes.
-use strict;
-
 sub gdriver {
   my($this, $options) = @_;
   print "GL gdriver...\n" if($PDL::Graphics::TriD::verbose);
@@ -555,22 +530,10 @@ sub gdriver {
      }
      $this->{_GLObject}->{winobjects}->[$this->{_GLObject}->{glutwindow}] = $this;      # circular ref
   }
-  print "gdriver: Calling glClearColor...\n" if ($PDL::Graphics::TriD::verbose);
+  print "gdriver: Calling glClearColor...\n" if $PDL::Graphics::TriD::verbose;
   glClearColor(0,0,0,1);
-  print "gdriver: Calling glpRasterFont...\n" if ($PDL::Graphics::TriD::verbose);
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
-     print STDERR "gdriver: window_type => 'glut' so not actually setting the rasterfont\n" if ($PDL::Graphics::TriD::verbose);
-     eval '$PDL::Graphics::TriD::GL::fontbase = GLUT_BITMAP_8_BY_13';
-  } else {
-     # NOTE: glpRasterFont() will die() if the requested font cannot be found
-     #       The new POGL+GLUT TriD implementation uses the builtin GLUT defined
-     #       fonts and does not have this failure mode.
-     my $lb =  eval { $this->{_GLObject}->glpRasterFont( ($ENV{PDL_3D_FONT} or "5x8"), 0, 256 ) };
-     if ( $@ ) {
-        die "glpRasterFont: unable to load font '%s', please set PDL_3D_FONT to an existing X11 font.";
-     }
-     $PDL::Graphics::TriD::GL::fontbase = $lb
-  }
+  print "gdriver: Calling glpRasterFont...\n" if $PDL::Graphics::TriD::verbose;
+  $PDL::Graphics::TriD::GL::fontbase = $this->{_GLObject}->glpRasterFont($ENV{PDL_3D_FONT} || "5x8", 0, 256);
   glShadeModel(GL_FLAT);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_NORMALIZE);
@@ -580,7 +543,7 @@ sub gdriver {
   my $light = pack "f*",1.0,1.0,1.0,0.0;
   glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
   glColor3f(1,1,1);
-  print "STARTED OPENGL!\n" if($PDL::Graphics::TriD::verbose);
+  print "STARTED OPENGL!\n" if $PDL::Graphics::TriD::verbose;
   if($PDL::Graphics::TriD::offline) {
     $this->doconfig($options->{width}, $options->{height});
   }
@@ -608,10 +571,7 @@ sub reshape {
 	}
 }
 
-sub get_size {
-  my $this=shift;
-  return ($this->{Width},$this->{Height});
-}
+sub get_size { @{$_[0]}{qw(Width Height)} }
 
 sub twiddle {
   my($this,$getout,$dontshow) = @_;
@@ -631,11 +591,6 @@ sub twiddle {
    print "EVENT!\n" if($PDL::Graphics::TriD::verbose);
 	 my $hap = 0;
 	 my $gotev = 0;
-         # Run a MainLoop event if GLUT windows
-         # this pumps the system allowing callbacks to populate
-         # the fake XEvent queue.
-         #
-         glutMainLoopEvent() if $this->{_GLObject}->{window_type} eq 'glut' and not $this->{_GLObject}->XPending();
          if ($this->{_GLObject}->XPending() or !$getout) {
             @e = $this->{_GLObject}->glpXNextEvent();
             $gotev=1;
@@ -714,10 +669,7 @@ sub domotion {
 sub display {
   my($this) = @_;
   return unless defined($this);
-  # set GLUT context to current window (for multiwindow support)
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {
-     glutSetWindow($this->{_GLObject}->{glutwindow});
-  }
+  $this->{_GLObject}->set_window; # for multiwindow support
   print "display: calling glClear()\n" if ($PDL::Graphics::TriD::verbose);
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
@@ -731,14 +683,8 @@ sub display {
 	 $vp->gl_call_list();
 	 glPopMatrix();
   }
-  print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
-  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
-    glutSwapBuffers();
-  } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
-    $this->{_GLObject}->glXSwapBuffers();
-  } else {
-    print "display: got object with inconsistent _GLObject info\n";
-  }
+  $this->{_GLObject}->swap_buffers;
+  print "display: after SwapBuffers\n" if $PDL::Graphics::TriD::verbose;
 }
 
 # should this really be in viewport?
@@ -760,22 +706,19 @@ package PDL::Graphics::TriD::EventHandler;
 
 use OpenGL qw(
   ConfigureNotify MotionNotify DestroyNotify
-  ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask
+  ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask Button4Mask
 );
 use PDL::Graphics::OpenGL::Perl::OpenGL;
 
 use fields qw/X Y Buttons VP/;
-use strict;
 sub new {
   my $class = shift;
   my $vp = shift;
-  no strict 'refs';
   my $self = fields::new($class);
   $self->{X} = -1;
   $self->{Y} = -1;
   $self->{Buttons} = [];
   $self->{VP} = $vp;
-
   $self;
 }
 
@@ -789,6 +732,7 @@ sub event {
 		$but = 0, last SWITCH if ($args[0] & (Button1Mask));
 		$but = 1, last SWITCH if ($args[0] & (Button2Mask));
 		$but = 2, last SWITCH if ($args[0] & (Button3Mask));
+		$but = 3, last SWITCH if ($args[0] & (Button4Mask));
 		print "No button pressed...\n" if($PDL::Graphics::TriD::verbose);
 		goto NOBUT;
 	 }
@@ -863,12 +807,12 @@ sub highlight {
 
 sub do_perspective {
 	my($this) = @_;
-	print "do_perspective ",$this->{W}," ",$this->{H} ,"\n" if($PDL::Graphics::TriD::verbose);
+	print "do_perspective ",$this->{W}," ",$this->{H} ,"\n" if $PDL::Graphics::TriD::verbose;
 	print Carp::longmess() if $PDL::Graphics::TriD::verbose>1;
         unless($this->{W}>0 and $this->{H}>0) {return;}
 	$this->{AspectRatio} = (1.0*$this->{W})/$this->{H};
 	glViewport($this->{X0},$this->{Y0},$this->{W},$this->{H});
-	$this->highlight() if($this->{Active});
+	$this->highlight if $this->{Active};
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	gluPerspective(40.0, $this->{AspectRatio} , 0.1, 200000.0);
